/*
 *	@(#)Cylinder.java 1.3 98/02/20 14:29:51
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import javax.media.j3d.*;
import javax.vecmath.*;

public class Cylinder {
   
   float verts[];
   float normals[];
   QuadArray quad = null;
   float div = 3.0f;
   Shape3D shape;

   public Cylinder(float x, float z, float radius, float length, int quality, Appearance a) {

      if (quality < 3) quality = 3;

      div = (float) quality;
      
      verts = new float[quality*12];
      normals = new float[quality*12];

      double inc = 2.0*Math.PI/(double)div;
      for (int i=0; i< quality; i++){
	 float z1 = radius * (float)Math.sin((double)i*inc) + z;
	 float x1 = radius * (float)Math.cos((double)i*inc) + x;
	 float z2 = radius * (float)Math.sin((double)(i+1)*inc) + z;
	 float x2 = radius * (float)Math.cos((double)(i+1)*inc) + x;

	 verts[12*i]    = x1;
	 verts[12*i+1]  = -length/2.f;
	 verts[12*i+2]  = z1;
	 verts[12*i+3]  = x1;
	 verts[12*i+4]  = length/2.f;
	 verts[12*i+5]  = z1;
	 verts[12*i+6]  = x2;
	 verts[12*i+7]  = length/2.f;
	 verts[12*i+8]  = z2;
	 verts[12*i+9]  = x2;
	 verts[12*i+10] = -length/2.f;
	 verts[12*i+11] = z2;
	 
	 float nz1 = (float)Math.sin((double)i*inc);
	 float nx1 = (float)Math.cos((double)i*inc);
	 float nz2 = (float)Math.sin((double)(i+1)*inc);
	 float nx2 = (float)Math.cos((double)(i+1)*inc);

	 normals[12*i] = nx1;
	 normals[12*i+1] = 0.0f;
	 normals[12*i+2] = nz1;
	 normals[12*i+3] = nx1;
	 normals[12*i+4] = 0.0f;
	 normals[12*i+5] = nz1;
	 normals[12*i+6] = nx2;
	 normals[12*i+7] = 0.0f;
	 normals[12*i+8] = nz2;
	 normals[12*i+9] = nx2;
	 normals[12*i+10] = 0.0f;
	 normals[12*i+11] = nz2;
      }
      
      quad = new QuadArray(quality*4, QuadArray.COORDINATES |
                                      QuadArray.NORMALS );
      quad.setCoordinates(0, verts);
      quad.setNormals(0, normals);
      shape = new Shape3D(quad, a);
   }

   Shape3D getShape(){
      return shape;
   }
}
