/*
 *	@(#)ReverberateSound.java 1.4 98/12/02 13:38:07
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

/*
 * ReverberateSound
 *
 * Same as MoveSound except this calls UniverseBuilderJS to use the
 * JavaSoundMixer AudioDevice rather than the HolosketchMixer device.
 *
 * NOTE: To run this anywhere but the Solaris Eng Menlo Park network
 * the URL path must be set to the java3d/javaone directory.
 */

import java.applet.Applet;
import java.awt.BorderLayout;
import java.awt.event.*;
import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.geometry.ColorCube;
import com.sun.j3d.utils.universe.*;
import java.io.File;
import javax.media.j3d.*;
import javax.vecmath.*;

public class ReverberateSound extends Applet {

    // File name of sound sample
    private static String[] filename = new String[1];
    private static String path = null;
    private static int filenamesGiven = 0;
    private static boolean filenamesSet = false;

    public BranchGroup createSceneGraph() {
	// Create the root of the subgraph
	BranchGroup objRoot = new BranchGroup();

	// Create the transform group node and initialize it to the identity.
	// Enable the TRANSFORM_WRITE capability so that our behavior code
	// can modify it at runtime.  Add it to the root of the subgraph.
	TransformGroup objTrans = new TransformGroup();
	objTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
	objRoot.addChild(objTrans);

	// Create a simple shape leaf node and add it into the scene graph.
	objTrans.addChild(new ColorCube(0.4));

	// Create a new Behavior object that will perform the desired
	// operation on the specified transform object and add it into the
	// scene graph.
	Transform3D yAxis = new Transform3D();
        Alpha rotation = new Alpha(-1, Alpha.INCREASING_ENABLE,
                                   0, 0,
                                   20000, 0, 0,
                                   0, 0, 0);
        RotationInterpolator rotator =
            new RotationInterpolator(rotation,
                                     objTrans, yAxis,
                                     0.0f, (float) Math.PI*2.0f);
	BoundingSphere bounds =
	    new BoundingSphere(new Point3d(0.0,0.0,0.0), 100.0);
	rotator.setSchedulingBounds(bounds);
	objTrans.addChild(rotator);

        //
        // Create an AuralAttribute with reverb params set
        //
        Soundscape soundScape2 = new Soundscape();
        AuralAttributes attributes2 = new AuralAttributes();
        attributes2.setReverbOrder(6);
        attributes2.setCapability(AuralAttributes.ALLOW_REVERB_ORDER_WRITE);
        attributes2.setCapability(AuralAttributes.ALLOW_REVERB_DELAY_WRITE);
        attributes2.setCapability(AuralAttributes.ALLOW_REFLECTION_COEFFICIENT_WRITE);
        soundScape2.setApplicationBounds(bounds);
        soundScape2.setAuralAttributes(attributes2);
        objRoot.addChild(soundScape2);

        //
        // Create a sound node and add it to the scene graph
        //
        PointSound sound = new PointSound();
        sound.setCapability(PointSound.ALLOW_ENABLE_WRITE);
        sound.setCapability(PointSound.ALLOW_INITIAL_GAIN_WRITE);
        sound.setCapability(PointSound.ALLOW_SOUND_DATA_WRITE);
        sound.setCapability(PointSound.ALLOW_DURATION_READ);
        sound.setCapability(PointSound.ALLOW_POSITION_WRITE);
        sound.setCapability(PointSound.ALLOW_LOOP_WRITE);
        sound.setSchedulingBounds(bounds);

	objTrans.addChild(sound);
        //
	// Create a new Behavior object that will play the sound
	//
	AudioReverberate player = new AudioReverberate(sound, filename[0],
                 attributes2);
	player.setSchedulingBounds(bounds);
	objTrans.addChild(player);


	return objRoot;
    }

    public ReverberateSound() {
 
        if (!filenamesSet) {
            // path is null if started from appletviewer
            // or no sound files were specified on command line.
            /*
             * and put into 'path' that contains URL-friendly "file:"
             * a null path shouldn't happen, but incase 'path' left NULL
             */
            if (path != null) {
                path = "file:" + path;
            }
            else { // no parameters were given, or running in appletviewer
                /*
                 *  The path below MUST be changed to run this program in
                 *  your environment.
                 */  
                path = new String("file:/net/java3d/export/java3d/javaone/data/sounds/");
                /**
                 ** Important note about path string:
                 **
                 **    At least for the time being, if this is to be run under
                 **    the Appletviewer, the path should point to a LOCAL directory
                 **    Otherwise an AccessControlException: access denied (due to
                 **    FilePermission) will be thrown.
                 **/
            }
    
            /*
             * append given file name to given URL path
             */
            if (filenamesGiven > 0) {
                filename[0] = new String(path + File.separator + filename[0]);
            }
            else {
                // fill in default file names if all three not given
                filename[0] = new String(path + File.separator + "hello_universe.au");
            }
            filenamesSet = true;
        }

	setLayout(new BorderLayout());
	Canvas3D c = new Canvas3D(null);
	add("Center", c);

        /*
         * Create a simple scene and attach it to the virtual universe
         */
        SimpleUniverse u = new SimpleUniverse(c);
        AudioDevice audioDev = u.getViewer().createAudioDevice();
        BranchGroup scene = createSceneGraph();

        // This will move the ViewPlatform back a bit so the
        // objects in the scene can be viewed.
        u.getViewingPlatform().setNominalViewingTransform();

	u.addBranchGraph(scene);
    }

    //
    // The following allows ReverberateSound to be run as an application
    // as well as an applet
    //
    public static void main(String[] args) {
        if (args.length > 0)
            path = args[0];

        if (args.length > 1) {
            filename[0] = args[1];
            if (filename[0] != null) {
                filenamesGiven++ ;
            }
        }
	new MainFrame(new ReverberateSound(), 256, 256);
    }
}
