/*
 *	@(#)SimplerSound.java 1.1 99/02/11 08:46:42
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import java.applet.Applet;
import java.net.URL;
import java.awt.BorderLayout;
import java.awt.event.*;
import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.geometry.ColorCube;
import com.sun.j3d.utils.universe.*;
import java.io.File;
import java.security.*;
import javax.media.j3d.*;
import javax.vecmath.*;

/*
 * This Java3D program:
 *    Creates an instance of the JavaSoundMixer AudioDevice, initializing it
 *         and attaching it to the PhysicalEnvironment by using
 *         SimpleUniverse.
 *    Creates one uncached Background sound source.
 *    Starts the sound playing
 *
 * IMPORTANT NOTE:
 *    Because of a "feature" of java.security the current directory
 *    can not longer be determined using File(".") call.
 *    Furthermore, for this program to work using appletviewer the 
 *    'path' directory MUST be a directory LOCAL to you machine.
 *    Trying to access a remote directory (such as /net/java3d/export/java3d...
 *    give a security access exception.
 *    See comments below.
 *
 * Usage: 
 *    java SimplerSound [URLpath [name1]]
 *
 * The first optional command line parameter is the URL path to directory
 *       containing "file:" or "http:" and then directory path string.
 *    If you are using the suppled default sound files in the same directory
 *       as this test program then only the URLpath need be supplied on the 
 *       command line.
 *    If this parameter is not included then the current path to the directory
 *       this program is running in is used.
 * The second thru fourth optional command line parameters are sound file names
 *    If not given, the default file names are:
 *       techno_machine.au
 *       hello_universe.au
 *       roar.au
 *    that correspond to the 3 'voice' quality, 8-bit, u-law, 8-kHz samples
 *    included in the same directory as this test program.
 * 
 *    Java Sound engine has been advertised to support the following 8- and 16-
 *    bit, linear and u-law, mono and stereo sound sample file formats: AU,
 *    AIFF, WAV, and PCM.  Non-cached MIDI and RMF files are also supported.
 *    Neither compressed formats (DVI, GSM, MOD) nor A-law formated files are
 *    supported at this time.
 */

public class SimplerSound extends Applet {

    // File name of sound sample
    private static  int filenamesGiven = 0;
    private static URL url = null;
    private static String filename = null;
    private static String path = null;
    private static boolean filenamesSet = false;

    public BranchGroup createSceneGraph() {
	// Create the root of the subgraph
	BranchGroup objRoot = new BranchGroup();

	// Create the transform group node and initialize it to the identity.
	// Enable the TRANSFORM_WRITE capability so that our behavior code
	// can modify it at runtime.  Add it to the root of the subgraph.
	TransformGroup objTrans = new TransformGroup();
	objTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
	objRoot.addChild(objTrans);

	// Create a simple shape leaf node and add it into the scene graph.
	objTrans.addChild(new ColorCube(0.4));

        /*
         * Create a sound node and add it to the scene graph
         */
        BackgroundSound sound = new BackgroundSound();
        sound.setCapability(PointSound.ALLOW_ENABLE_WRITE);
        MediaContainer sample  = new MediaContainer();
        sample.setURL(url);
        sample.setCacheEnable(true);  // sound must be cached in order to loop
        sound.setLoop(5);
        sound.setContinuousEnable(false);
        sound.setReleaseEnable(false);
        sound.setSoundData(sample);
        sound.setInitialGain(0.7f);
	BoundingSphere soundBounds =
	    new BoundingSphere(new Point3d(0.0,0.0,0.0), 100.0);
        sound.setSchedulingBounds(soundBounds);
	objTrans.addChild(sound);

        /*
	 * Start the Sound playing
	 */

        sound.setEnable(true);

        // Let Java 3D perform optimizations on this scene graph.
        objRoot.compile();

	return objRoot;
    }

    public SimplerSound() {
        if (!filenamesSet) {
            /*
             * append given file name to given URL path
             */
            // path is null if started from appletviewer/browser the first time
            // or no sound files were specified on command line.
            /*
             * and put into 'path' that contains URL-friendly "file:"
             * a null path shouldn't happen, but incase 'path' left NULL
             */
            if (path != null) {
                path = "file:" + path;
            }
            else { // no parameters were given, or running in appletviewer
                /*
                 *  The path below MUST be changed to run this program in
                 *  your environment.
                 */
                path = "file:/net/java3d/export/java3d/javaone/data/sounds";
                /**
                 ** Important note about path string:
                 **
                 **    At least for the time being, if this is to be run under
                 **    the Appletviewer, the path should point to a LOCAL directory
                 **    Otherwise an AccessControlException: access denied (due to
                 **    FilePermission) will be thrown. 
                 **/
            }
            int j;
            if (filenamesGiven == 1)
                filename = new String(path + File.separator + filename);
            else {
                /*
                 * fill in default file names if all three not given
                 */
                filename = new String(path + File.separator +
                                                        "techno_machine.au");
            }
            filenamesSet = true;
        }

	setLayout(new BorderLayout());
	Canvas3D c = new Canvas3D(null);
	add("Center", c);

        /*
         * Change filenames into URLs
         */
        try {
            url = new URL(filename);
        }
        catch (Exception e) {
            // Local directory can no longer be retrieved by doing a
            // File(".") due to security feature of JDK 1.2 .
            System.out.println(
                "Path to sounds MUST be changed to reference local files");
            return;
        }

	/*
         * Create a simple scene and attach it to the virtual universe
         */
	SimpleUniverse u = new SimpleUniverse(c);
        AudioDevice audioDev = u.getViewer().createAudioDevice();
	BranchGroup scene = createSceneGraph();

        // This will move the ViewPlatform back a bit so the
        // objects in the scene can be viewed.
        u.getViewingPlatform().setNominalViewingTransform();

	u.addBranchGraph(scene);
    }

    /*
     * The following allows SimplerSound to be run as an application
     * as well as an applet
     */
    public static void main(String[] args) {
        if (args.length > 0) 
            path = args[0];
        if (args.length > 1) {
            filename = args[1];
            filenamesGiven++ ;
        }
	new MainFrame(new SimplerSound(), args, 256, 256);
    }
}
