/*
 * support/include/nfslib.h
 *
 * General support functions for NFS user-space programs.
 *
 * Copyright (C) 1995 Olaf Kirch <okir@monad.swb.de>
 */

#ifndef NFSLIB_H
#define NFSLIB_H

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <stdio.h>
#include <paths.h>
#include <rpcsvc/nfs_prot.h>
#include <nfs/nfs.h>
#include "xlog.h"

#ifndef _PATH_EXPORTS
#define _PATH_EXPORTS		"/etc/exports"
#endif
#ifndef _PATH_XTAB
#define _PATH_XTAB		"/var/lib/nfs/xtab"
#endif
#ifndef _PATH_XTABTMP
#define _PATH_XTABTMP		"/var/lib/nfs/xtab.tmp"
#endif
#ifndef _PATH_RMTAB
#define _PATH_RMTAB		"/var/lib/nfs/rmtab"
#endif
#ifndef _PATH_RMTABTMP
#define _PATH_RMTABTMP		_PATH_RMTAB ".tmp"
#endif

enum cle_maptypes {
	CLE_MAP_IDENT = 0,
	CLE_MAP_FILE,
	CLE_MAP_UGIDD,
};

/*
 * Data related to a single exports entry as returned by getexportent.
 * FIXME: export options should probably be parsed at a later time to 
 * allow overrides when using exportfs.
 */
struct exportent {
	char		e_hostname[NFSCLNT_IDMAX+1];
	char		e_path[NFS_MAXPATHLEN+1];
	/* The mount path may be different from the exported path due
	   to submount. It may change for every mount. The idea is we
	   set m_path every time when we process a mount. We should not
	   use it for anything else. */
	char		m_path[NFS_MAXPATHLEN+1];
	int		e_flags;
	int		e_maptype;
	int		e_anonuid;
	int		e_anongid;
	int *		e_squids;
	int		e_nsquids;
	int *		e_sqgids;
	int		e_nsqgids;
};

struct rmtabent {
	char		r_client[NFSCLNT_IDMAX+1];
	char		r_path[NFS_MAXPATHLEN+1];
};

/*
 * configuration file parsing
 */
void			setexportent(char *fname, char *type);
struct exportent *	getexportent(void);
void			putexportent(struct exportent *xep);
void			endexportent(void);
struct exportent *	mkexportent(char *hname, char *path, char *opts);
void			dupexportent(struct exportent *dst,
					struct exportent *src);
int			updateexportent(struct exportent *eep, char *options);

int			setrmtabent(char *type);
struct rmtabent *	getrmtabent(int log);
void			putrmtabent(struct rmtabent *xep);
void			endrmtabent(void);
void			rewindrmtabent(void);
FILE *			fsetrmtabent(char *fname, char *type);
struct rmtabent *	fgetrmtabent(FILE *fp, int log);
void			fputrmtabent(FILE *fp, struct rmtabent *xep);
void			fendrmtabent(FILE *fp);
void			frewindrmtabent(FILE *fp);

/*
 * wildmat borrowed from INN
 */
int			wildmat(char *text, char *pattern);

/*
 * nfsd library functions.
 */
int			nfsctl(int, struct nfsctl_arg *, union nfsctl_res *);
int			nfssvc(int port, int nrservs);
int			nfsaddclient(struct nfsctl_client *clp);
int			nfsdelclient(struct nfsctl_client *clp);
int			nfsexport(struct nfsctl_export *exp);
int			nfsunexport(struct nfsctl_export *exp);
struct knfs_fh *	getfh_old(struct sockaddr *addr, dev_t dev, ino_t ino);
struct knfs_fh *	getfh(struct sockaddr *addr, const char *);

#endif /* NFSLIB_H */
