/*
 *	Ohio Trollius
 *	Copyright 1997 The Ohio State University
 *	NJN
 *
 *	$Id: mpi_types.h,v 6.2 1997/08/24 18:04:24 lam Exp $
 *
 *	Function:	- type conversions for ANSI C types and alignment
 *			  checking
 */

#ifndef _MPI_TYPES_H
#define _MPI_TYPES_H

#include <lam_config.h>
#include <mpi.h>
#include <t_types.h>

#if WORDS_BIGENDIAN

#define	ttol_short(x, y)	(*((short *) (y)) = *((short *) (x)))
#define	ltot_short(x, y)	(*((short *) (y)) = *((short *) (x)))
#define	mttol_short(x, l)
#define	mltot_short(x, l)

#define	ttol_int(x, y)		(*((int *) (y)) = *((int *) (x)))
#define	ltot_int(x, y)		(*((int *) (y)) = *((int *) (x)))
#define	mttol_int(x, l)
#define	mltot_int(x, l)

#define	ttol_wchar(x, y)	(*((wchar_t *) (y)) = *((wchar_t *) (x)))
#define	ltot_wchar(x, y)	(*((wchar_t *) (y)) = *((wchar_t *) (x)))
#define	mttol_wchar(x, l)
#define	mltot_wchar(x, l)

#define	ttol_long(x, y)		(*((long *) (y)) = *((long *) (x)))
#define	ltot_long(x, y)		(*((long *) (y)) = *((long *) (x)))
#define	mttol_long(x, l)
#define	mltot_long(x, l)

#if SIZEOF_LONG_LONG
#define	ttol_longlong(x, y)	(*((long long *) (y)) = *((long long *) (x)))
#define	ltot_longlong(x, y)	(*((long long *) (y)) = *((long long *) (x)))
#define	mttol_longlong(x, l)
#define	mltot_longlong(x, l)
#endif

#define	ttol_float(x, y)	(*((float *) (y)) = *((float *) (x)))
#define	ltot_float(x, y)	(*((float *) (y)) = *((float *) (x)))
#define	mttol_float(x, l)
#define	mltot_float(x, l)

#define	ttol_double(x, y)	(*((double *) (y)) = *((double *) (x)))
#define	ltot_double(x, y)	(*((double *) (y)) = *((double *) (x)))
#define	mttol_double(x, l)
#define	mltot_double(x, l)

#if SIZEOF_LONG_DOUBLE
#define ttol_longdbl(x, y)	(*((long double*) (y)) = *((long double*) (x)))
#define ltot_longdbl(x, y)	(*((long double*) (y)) = *((long double*) (x)))
#define	mttol_longdbl(x, l)
#define	mltot_longdbl(x, l)
#endif

#else  /* not WORDS_BIGENDIAN */

#if SIZEOF_SHORT == 2
#define	ttol_short(x, y)	revn(x, y, 2)
#define	ltot_short(x, y)	revn(x, y, 2)
#define	mttol_short(x, l)	mrevn(x, 2, l)
#define	mltot_short(x, l)	mrevn(x, 2, l)
#elif SIZEOF_SHORT == 4
#define	ttol_short(x, y)	rev4(x, y)
#define	ltot_short(x, y)	rev4(x, y)
#define	mttol_short(x, l)	mrev4(x, l)
#define	mltot_short(x, l)	mrev4(x, l)
#endif

#if SIZEOF_INT == 4
#define	ttol_int(x, y)		rev4(x, y)
#define	ltot_int(x, y)		rev4(x, y)
#define	mttol_int(x, l)		mrev4(x, l)
#define	mltot_int(x, l)		mrev4(x, l)
#elif SIZEOF_INT == 8
#define	ttol_int(x, y)		rev8(x, y)
#define	ltot_int(x, y)		rev8(x, y)
#define	mttol_int(x, l)		mrev8(x, l)
#define	mltot_int(x, l)		mrev8(x, l)
#endif

#if SIZEOF_WCHAR == 4
#define	ttol_wchar(x, y)	rev4(x, y)
#define	ltot_wchar(x, y)	rev4(x, y)
#define	mttol_wchar(x, l)	mrev4(x, l)
#define	mltot_wchar(x, l)	mrev4(x, l)
#endif

#if SIZEOF_LONG == 4
#define	ttol_long(x, y)		rev4(x, y)
#define	ltot_long(x, y)		rev4(x, y)
#define	mttol_long(x, l)	mrev4(x, l)
#define	mltot_long(x, l)	mrev4(x, l)
#elif SIZEOF_LONG == 8
#define	ttol_long(x, y)		rev8(x, y)
#define	ltot_long(x, y)		rev8(x, y)
#define	mttol_long(x, l)	mrev8(x, l)
#define	mltot_long(x, l)	mrev8(x, l)
#endif

#if SIZEOF_LONG_LONG == 8
#define	ttol_longlong(x, y)	rev8(x, y)
#define	ltot_longlong(x, y)	rev8(x, y)
#define	mttol_longlong(x, l)	mrev8(x, l)
#define	mltot_longlong(x, l)	mrev8(x, l)
#elif SIZEOF_LONG_LONG
#define	ttol_longlong(x, y)	revn(x, y, SIZEOF_LONG_LONG)
#define	ltot_longlong(x, y)	revn(x, y, SIZEOF_LONG_LONG)
#define	mttol_longlong(x, l)	mrevn(x, SIZEOF_LONG_LONG, l)
#define	mltot_longlong(x, l)	mrevn(x, SIZEOF_LONG_LONG, l)
#endif

#if SIZEOF_FLOAT == 4
#define	ttol_float(x, y)	rev4(x, y)
#define	ltot_float(x, y)	rev4(x, y)
#define	mttol_float(x, l)	mrev4(x, l)
#define	mltot_float(x, l)	mrev4(x, l)
#elif SIZEOF_FLOAT == 8
#define	ttol_float(x, y)	rev8(x, y)
#define	ltot_float(x, y)	rev8(x, y)
#define	mttol_float(x, l)	mrev8(x, l)
#define	mltot_float(x, l)	mrev8(x, l)
#endif

#if SIZEOF_DOUBLE == 8
#define	ttol_double(x, y)	rev8(x, y)
#define	ltot_double(x, y)	rev8(x, y)
#define	mttol_double(x, l)	mrev8(x, l)
#define	mltot_double(x, l)	mrev8(x, l)
#elif SIZEOF_DOUBLE
#define	ttol_double(x, y)	revn(x, y, SIZEOF_DOUBLE)
#define	ltot_double(x, y)	revn(x, y, SIZEOF_DOUBLE)
#define	mttol_double(x, l)	mrevn(x, SIZEOF_DOUBLE, l)
#define	mltot_double(x, l)	mrevn(x, SIZEOF_DOUBLE, l)
#endif

#if SIZEOF_LONG_DOUBLE == 8
#define	ttol_longdbl(x, y)	rev8(x, y)
#define	ltot_longdbl(x, y)	rev8(x, y)
#define	mttol_longdbl(x, l)	mrev8(x, l)
#define	mltot_longdbl(x, l)	mrev8(x, l)
#elif SIZEOF_LONG_DOUBLE
#define	ttol_longdbl(x, y)	revn(x, y, SIZEOF_LONG_DOUBLE)
#define	ltot_longdbl(x, y)	revn(x, y, SIZEOF_LONG_DOUBLE)
#define	mttol_longdbl(x, l)	mrevn(x, SIZEOF_LONG_DOUBLE, l)
#define	mltot_longdbl(x, l)	mrevn(x, SIZEOF_LONG_DOUBLE, l)
#endif


#endif


/*
 * alignment checking macros
 */

#if SIZEOF_INT == 4
#define ALIGNEDU4(p)	((((MPI_Aint) (p)) & (ALIGNMENT_INT - 1)) == 0)
#elif SIZEOF_SHORT == 4
#define ALIGNEDU4(p)	((((MPI_Aint) (p)) & (ALIGNMENT_SHORT - 1)) == 0)
#elif SIZEOF_LONG == 4
#define ALIGNEDU4(p)	((((MPI_Aint) (p)) & (ALIGNMENT_LONG - 1)) == 0)
#endif

#if SIZEOF_FLOAT == 8
#define ALIGNEDF8(p)	((((MPI_Aint) (p)) & (ALIGNMENT_FLOAT - 1)) == 0)
#elif SIZEOF_DOUBLE == 8
#define ALIGNEDF8(p)	((((MPI_Aint) (p)) & (ALIGNMENT_DOUBLE - 1)) == 0)
#endif


#ifndef __ARGS
#if __STDC__ || defined(c_plusplus) || defined(__cplusplus)
#define __ARGS(a)	a
#else
#define __ARGS(a)	()
#endif
#endif

#ifdef __cplusplus
extern "C" {
#endif
	
extern void		lam_copyrev2 __ARGS((void *, void *, int));
extern void		lam_copyrev4 __ARGS((void *, void *, int));
extern void		lam_copyrev8 __ARGS((void *, void *, int));
extern void		lam_copyrevn __ARGS((void *, void *, int, int));

#ifdef __cplusplus
}
#endif

#endif /* _MPI_TYPES_H */
