/*
 *	Ohio Trollius
 *	Copyright 1997 The Ohio State University
 *	RBD/NJN
 *
 *	$Id: cdelattr.c,v 0.2 1998/03/16 00:43:19 lam Exp $
 *
 *	Function:	- delete attribute
 *			- detach attribute from communicator
 *	Accepts:	- communicator
 *			- attribute key
 *	Returns:	- MPI_SUCCESS or error code
 */

#include <errno.h>

#include <blktype.h>
#include <mpi.h>
#include <mpisys.h>


int
MPI_Comm_delete_attr(comm, key)

MPI_Comm		comm;
int			key;

{
	struct _attrkey	*p;
	struct _attr	*pk;
	int		err;

	lam_initerr_m();
	lam_setfunc_m(BLKMPICOMMDELETEATTR);
/*
 * Check the arguments.
 */
	if (comm == MPI_COMM_NULL) {
		return(lam_err_comm(MPI_COMM_WORLD,
				MPI_ERR_COMM, 0, "null handle"));
	}

	p = lam_getattr(key);
	if (p == 0) {
		return(lam_err_comm(comm, MPI_ERR_KEYVAL, 0, ""));
	}
/*
 * Check if the key is attached to the communicator.
 */
	pk = lam_getkey(comm->c_keys, key);
	if (pk == 0) {
		return(lam_err_comm(comm, MPI_ERR_KEYVAL, 0,
			"not on communicator"));
	}
/*
 * Delete the attribute via the callback function.
 */
	if (((MPI_Comm_delete_attr_function *) (p->ak_del))
			!= MPI_COMM_NULL_DELETE_FN) {

		if (p->ak_flags & LAM_LANGF77) {
			(*((MPI_F_delete_function *) (p->ak_del)))
				(&comm->c_f77handle, &key,
				(int *) &(pk->a_value),
				(int *) &(p->ak_extra), &err);
		} else {
			err = (*((MPI_Comm_delete_attr_function *) (p->ak_del)))
				(comm, key, pk->a_value, p->ak_extra);
		}

		if (err != MPI_SUCCESS) {
			return(lam_err_comm(comm, err, 0,
					"deleting attribute"));
		}
	}
/*
 * Detach the key from the communicator.
 */
	if (lam_delkey(comm->c_keys, key)) {
		return(lam_err_comm(comm, MPI_ERR_INTERN, errno,
			"detaching key"));
	}

	lam_resetfunc_m(BLKMPICOMMDELETEATTR);
	return(MPI_SUCCESS);
}
