/*
 *	Ohio Trollius
 *	Copyright 1997 The Ohio State University
 *	RBD/NJN
 *
 *	$Id: cgetattr.c,v 0.2 1998/03/16 00:44:05 lam Exp $
 *
 *	Function:	- get communicator attribute value
 *	Accepts:	- communicator
 *			- attribute key
 *			- attribute value (out)
 *			- found flag (out)
 *	Returns:	- MPI_SUCCESS or error code
 */

#include <blktype.h>
#include <mpi.h>
#include <mpisys.h>


int
MPI_Comm_get_attr(comm, key, value, found)

MPI_Comm		comm;
int			key;
void			*value;
int			*found;

{
        struct _attrkey *pk;
	struct _attr    *p;

	lam_initerr_m();
	lam_setfunc_m(BLKMPICOMMGETATTR);
/*
* Check the arguments.
*/
	if (comm == MPI_COMM_NULL) {
		return(lam_err_comm(MPI_COMM_WORLD, MPI_ERR_COMM, 0,
					"null handle"));
	}

	if ((value == 0) || (found == 0)) {
		return(lam_err_comm(comm, MPI_ERR_ARG, 0, "null pointer"));
	}
	
	pk = lam_getattr(key);
	if (pk == 0) {
		return(lam_err_comm(comm, MPI_ERR_KEYVAL, 0, ""));
	}

	if (!(pk->ak_flags & LAM_AKCOMM)) {
		return(lam_err_comm(comm, MPI_ERR_KEYVAL, 0,
					"not a communicator key"));
	}
/*
 * Check if the key is attached to the communicator.
 */
	p = lam_getkey(comm->c_keys, key);
	if (p == 0) {
		*found = 0;
	} else {
		*found = 1;
		*((void **) value) = (pk->ak_flags & LAM_LANGF77)
			? (void *) &p->a_value : p->a_value;
	}

	lam_resetfunc_m(BLKMPICOMMGETATTR);
	return(MPI_SUCCESS);
}
