/*
 *	Ohio Trollius
 *	Copyright 1996 The Ohio State University
 *	RBD
 *
 *	$Id: errfree.c,v 6.2 1998/05/26 05:02:15 lam Exp $
 *
 *	Function:	- free error handler
 *	Accepts:	- ptr error handler
 *	Returns:	- MPI_SUCCESS or error code
 */

#include <stdlib.h>

#include <blktype.h>
#include <mpi.h>
#include <mpisys.h>
#include <MPISYS.h>

int
MPI_Errhandler_free(errhdl)

MPI_Errhandler		*errhdl;

{
	lam_initerr_m();
	lam_setfunc_m(BLKMPIERRFREE);
/*
 * Check the arguments.
 */
	if (errhdl == 0 || *errhdl == MPI_ERRHANDLER_NULL) {
		return(lam_errfunc(MPI_COMM_WORLD,
			BLKMPIERRFREE, lam_mkerr(MPI_ERR_ARG, 0)));
	}
/*
 * Only internal library calls can free predefined handlers.
 */
	if (lam_getfunc() == BLKMPIERRFREE && (*errhdl == MPI_ERRORS_ARE_FATAL
			|| *errhdl == MPI_ERRORS_RETURN)) {
		return(lam_errfunc(MPI_COMM_WORLD,
			BLKMPIERRFREE, lam_mkerr(MPI_ERR_ARG, 0)));
	}
/*
 * Free the handler.
 */
	if (--((*errhdl)->eh_refcount) == 0) {

		if (*errhdl == MPI_ERRORS_ARE_FATAL
				|| *errhdl == MPI_ERRORS_RETURN) {
			return(lam_errfunc(MPI_COMM_WORLD,
				BLKMPIERRFREE, lam_mkerr(MPI_ERR_INTERN, 0)));
		}

		if ((*errhdl)->eh_f77handle >= 0) {
			lam_F_free_hdl((*errhdl)->eh_f77handle);
		}

		free((char *) *errhdl);
	}

	*errhdl = MPI_ERRHANDLER_NULL;

	lam_resetfunc_m(BLKMPIERRFREE);
	return(MPI_SUCCESS);
}
