/*
 *	Ohio Trollius
 *	Copyright 1997 The Ohio State University
 *	NJN
 *
 *	$Id: infoget.c,v 6.2 1998/05/26 05:02:15 lam Exp $
 *
 *	Function:	- get info value associated with key 
 *	Accepts:	- info
 *			- key
 *			- maximum length of value
 *			- value (returned)
 *			- key found flag (returned)
 *	Returns:	- MPI_SUCCESS or error code
 */

#include <string.h>

#include <all_list.h>
#include <blktype.h>
#include <mpi.h>
#include <mpisys.h>
#include <terror.h>


int
MPI_Info_get(info, key, valuelen, value, flag)

MPI_Info		info;
char			*key;
int			valuelen;
char			*value;
int			*flag;

{
	struct _infoent	search;			/* search element */
	struct _infoent	*found;			/* found key/value pair */
	int		keylen;			/* key length */
	int		vallen;			/* stored value length */
	
	lam_initerr_m();
	lam_setfunc_m(BLKMPIINFOGET);

	if (info == 0 || value == 0 || valuelen <= 0) {
		return(lam_errfunc(MPI_COMM_WORLD, BLKMPIINFOGET,
					lam_mkerr(MPI_ERR_ARG, 0)));
	}

	keylen = (key) ? strlen(key) : 0;
	if (keylen == 0 || keylen > MPI_MAX_INFO_KEY) {
		return(lam_errfunc(MPI_COMM_WORLD, BLKMPIINFOGET,
					lam_mkerr(MPI_ERR_INFO_KEY, 0)));
	}

	strcpy(search.ie_key, key);

	if ((found = al_find(info->info_list, &search))) {
		*flag = 1;
		vallen = strlen(found->ie_value);
		if (vallen < valuelen) {
		    strcpy(value, found->ie_value);
		} else {
		    strncpy(value, found->ie_value, valuelen);
		    value[valuelen - 1] = 0;
		}
	} else {
		*flag = 0;
	}

	lam_resetfunc_m(BLKMPIINFOGET);
	return(MPI_SUCCESS);
}
