/*
 *	Ohio Trollius
 *	Copyright 1996 The Ohio State University
 *	RBD
 *
 *	$Id: lammisc.c,v 6.5 1998/09/29 10:11:23 lamteam Exp $
 *
 *	Function:	- miscellaneous functions
 */

#include <lam_config.h>

#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>

#if NEED_SYS_SELECT_H
#include <sys/select.h>
#endif

#include <mpi.h>
#include <mpisys.h>
#include <terror.h>

/*
 *	lam_cubedim
 *
 *	Function:	- get dimension of inscribing hypercube
 *	Accepts:	- # processes
 *	Returns:	- dimension or -1
 */
int
lam_cubedim(nprocs)

int			nprocs;

{
	int		dim;
	unsigned	size;

	if (nprocs < 1) return(-1);

	for (dim = 0, size = 1; size < nprocs; ++dim, size <<= 1);

	return(dim);
}

/*
 *	lam_hibit
 *
 *	Function:	- get position of the highest bit set
 *	Accepts:	- process rank
 *			- hypercube dimension
 *	Returns:	- position or -1
 */
int
lam_hibit(rank, dim)

int			rank;
int			dim;

{
	unsigned	mask;

	if ((rank < 1) || (dim < 1)) return(-1);

	--dim;
	mask = 1 << dim;

	for (; dim >= 0; --dim, mask >>= 1) {
		if (rank & mask) break;
	}

	return(dim);
}

/*
 *	lam_emtpystat
 *
 *	Function:	- set status to empty
 *	Accepts:	- ptr status
 */
void
lam_emptystat(pstat)

MPI_Status		*pstat;

{
	pstat->MPI_SOURCE = MPI_ANY_SOURCE;
	pstat->MPI_TAG = MPI_ANY_TAG;
	pstat->MPI_ERROR = MPI_ERR_PENDING;
	pstat->st_length = 0;
}

/*
 *      lam_yield
 *
 *      Function:       - make best effort to yield the processor
 */
void
lam_yield()
{
#if HAVE_YIELD && !USE_SELECT_YIELD

	yield();

#elif HAVE_SCHED_YIELD && !USE_SELECT_YIELD

	sched_yield();

#else

        struct timeval  timeout;

        timeout.tv_sec = 0;
        timeout.tv_usec = 1;

        select(0, (fd_set *) 0, (fd_set *) 0, (fd_set *) 0, &timeout);
#endif
}

