/*
 *	Ohio Trollius
 *	Copyright 1997 The Ohio State University
 *	NJN
 *
 *	$Id: lamsig.c,v 0.1 1997/08/24 20:55:52 lam Exp $
 *
 *	Function:	- exception signal catching
 */

#include <sys/signal.h>
#include <stdio.h>
#include <unistd.h>

#include <app_mgmt.h>
#include <blktype.h>
#include <mpi.h>
#include <mpisys.h>
#include <rpisys.h>
#include <portable.h>
#include <preq.h>
#include <priority.h>

typedef void 		(*sighandler_t)(int);

/*
 * public functions
 */
int			lam_mpi_set_sighandlers();
void			lam_mpi_reset_sighandlers();

/*
 * private functions
 */
static void		lam_mpi_sig_handler();

/*
 * external functions
 */
extern void		(*(_lam_signal()))();

/*
 * private variables
 */
static sighandler_t	old_segv_handler;
static sighandler_t	old_bus_handler;
static sighandler_t	old_fpe_handler;
static sighandler_t	old_ill_handler;


/*
 *	lam_mpi_set_sighandlers
 *
 *	Function:	- 
 *	Accepts:	-
 *	Returns:	- 
 */
int
lam_mpi_set_sighandlers()

{
	old_segv_handler = _lam_signal(SIGSEGV, lam_mpi_sig_handler);
	if (old_segv_handler == SIG_ERR) return(LAMERROR);

	old_bus_handler = _lam_signal(SIGBUS, lam_mpi_sig_handler);
	if (old_bus_handler == SIG_ERR) return(LAMERROR);

	old_fpe_handler = _lam_signal(SIGFPE, lam_mpi_sig_handler);
	if (old_fpe_handler == SIG_ERR) return(LAMERROR);

	old_ill_handler = _lam_signal(SIGILL, lam_mpi_sig_handler);
	if (old_ill_handler == SIG_ERR) return(LAMERROR);

	return(0);
}


/*
 *	lam_mpi_set_sighandlers
 *
 *	Function:	- 
 *	Accepts:	-
 *	Returns:	- 
 */
void
lam_mpi_reset_sighandlers()

{
	_lam_signal(SIGSEGV, old_segv_handler);
	_lam_signal(SIGBUS, old_bus_handler);
	_lam_signal(SIGFPE, old_fpe_handler);
	_lam_signal(SIGILL, old_ill_handler);
}

/*
 *	lam_mpi_sig_handler
 *
 *	Function:	- handler for exceptions
 *	Accepts:	- signal number
 */
static void
lam_mpi_sig_handler(sig)

int 			sig;

{
	struct _proc	**p;
	sighandler_t	old_handler;
	CONST char	*err;
	int		mpifn;
	int		i;

	switch (sig) {

	case SIGSEGV:
		old_handler = old_segv_handler;
		err = "SIGSEGV";
		break;
	case SIGBUS:
		old_handler = old_bus_handler;
		err = "SIGBUS"; 
		break;
	case SIGFPE:
		old_handler = old_fpe_handler;
		err = "SIGFPE";
		break;
	case SIGILL:
		old_handler = old_ill_handler;
		err = "SIGILL";
		break;
	default:
		return;
	}

	if (old_handler == SIG_IGN) return;

	if ((mpifn = lam_getfunc_m())) {
		fprintf(stderr,
			"MPI process rank %d (n%d, p%d) got %s in %s.\n",
			lam_myproc->p_gps.gps_grank,
			lam_myproc->p_gps.gps_node,
			lam_myproc->p_gps.gps_pid, err, blktype(mpifn));
	} else {
		fprintf(stderr,
			"MPI process rank %d (n%d, p%d) got %s.\n",
			lam_myproc->p_gps.gps_grank,
			lam_myproc->p_gps.gps_node,
			lam_myproc->p_gps.gps_pid, err);
	}
	fflush(stderr);

	if (((_kio.ki_rtf & RTF_MPIRUN) || (_kio.ki_parent > 0))
			&& MPI_COMM_WORLD->c_group) {

		if (fork() == 0) {
			if (kinit(PRCMD)) {
				exit(errno);
			}

			p = MPI_COMM_WORLD->c_group->g_procs;
			for (i = 0; i < MPI_COMM_WORLD->c_group->g_nprocs;
					++i, ++p) {

				rpdoom((*p)->p_gps.gps_node, SELECT_PID,
						(*p)->p_gps.gps_pid, SIGUDIE);
			}

			kexit(0);
		}
	}

	kexit(1);
}
