/*
 *	Ohio Trollius
 *	Copyright 1997 The Ohio State University
 *	RBD/GDB
 *
 *	$Id: lamtest.c,v 6.2 1998/05/26 05:02:15 lam Exp $
 *
 *	Function:	- test if request is done
 *			- destructive and non-destructive modes
 *	Accepts:	- request
 *			- destructive mode flag
 *			- flag (out)
 *			- status (out)
 *	Returns:	- MPI_SUCCESS or error code
 */

#include <mpi.h>
#include <mpisys.h>
#include <mpitrace.h>
#include <rpisys.h>
#include <terror.h>


int
lam_test(preq, destroy, flag, stat)

MPI_Request		*preq;
int			destroy;
int			*flag;
MPI_Status		*stat;

{
	MPI_Request	req;			/* request */
	int		errstat;		/* error in status */
	int		err;			/* error code */
	int		fl_trace;		/* do tracing? */
	double		startt;			/* start time */
	double		finisht;		/* finish time */

	if (preq == 0) return(lam_mkerr(MPI_ERR_REQUEST, 0));

	if (flag == 0) return(lam_mkerr(MPI_ERR_ARG, 0));

	if ((fl_trace = LAM_TRACE_TOP())) {
	    startt = MPI_Wtime();
	    _kio.ki_blktime = 0.0;
	}
/*
 * Advance the system.
 */
	_mpi_req_blkclr_m();

	err = _mpi_req_advance();
	if (err != MPI_SUCCESS) return(err);
/*
 * Inactive requests get an empty status.
 */
	req = *preq;

	if ((req == MPI_REQUEST_NULL) || (req->rq_state == LAM_RQSINIT)) {
	    *flag = 1;
	    if (stat != MPI_STATUS_IGNORE) {
		    lam_emptystat(stat);
	    }
	}
/*
 * If not done reset flag.
 */
	else if (req->rq_state != LAM_RQSDONE) {
	    *flag = 0;
	}
/*
 * Finish the request if in destructive-mode.
 */
	else {
	    *flag = 1;

	    if (destroy) {
		_mpi_req_rem_m(req);

		err = _mpi_req_end(req);
		if (err != MPI_SUCCESS) return(err);

		errstat = req->rq_status.MPI_ERROR;
		if (stat != MPI_STATUS_IGNORE) {
			*stat = req->rq_status;
		}
/*
 * Generate a run time trace except in the case of cancelled request or
 * a request where the peer is MPI_PROC_NULL.
 */
		if (fl_trace && (req->rq_rank != MPI_PROC_NULL)
				&& !(req->rq_flags & LAM_RQFCANCEL)) {
		    finisht = MPI_Wtime();

		    lam_tr_msg( (req->rq_type == LAM_RQIRECV)
			    ? TRTINPUT : TRTNOIO, startt,
			    LAM_S2US(finisht - startt - _kio.ki_blktime),
			    LAM_S2US(_kio.ki_blktime), req->rq_rank,
			    req->rq_tag, req->rq_comm, req->rq_dtype,
			    req->rq_count, req->rq_status.MPI_SOURCE,
			    req->rq_status.MPI_TAG, req->rq_seq, req->rq_type);
		}
/*
 * Reset persistent requests.
 * Destroy requests that are not persistent.
 */
		if (req->rq_marks & LAM_RQFPERSIST) {
		    req->rq_state = LAM_RQSINIT;
		} else {
		    err = _mpi_req_destroy(preq);
		    if (err != MPI_SUCCESS) return(err);
		}

		if (errstat != MPI_SUCCESS) {
		    return(errstat);
		}
	    }
	}
	
	return(MPI_SUCCESS);
}
