/*
 *	Ohio Trollius
 *	Copyright 1997 The Ohio State University
 *	RBD
 *
 *	$Id: recvinit.c,v 6.2 1998/05/26 05:02:15 lam Exp $
 *
 *	Function:	- create a receive request
 *	Accepts:	- buffer
 *			- count
 *			- datatype
 *			- source
 *			- tag
 *			- communicator
 *			- request (out)
 *	Returns: 	- MPI_SUCCESS or error code
 */

#include <blktype.h>
#include <mpi.h>
#include <mpisys.h>
#include <rpisys.h>


int
MPI_Recv_init(buf, count, dtype, src, tag, comm, req)

void			*buf;
int			count;
MPI_Datatype		dtype;
int			src;
int			tag;
MPI_Comm		comm;
MPI_Request		*req;

{
	int		err;			/* error code */

	lam_initerr_m();
	lam_setfunc_m(BLKMPIRINIT);

	if ((tag < 0 && tag != MPI_ANY_TAG) || tag > LAM_MAXTAG) {
		return(lam_errfunc(comm, BLKMPIRINIT,
				lam_mkerr(MPI_ERR_TAG, 0)));
	}

	*req = MPI_REQUEST_NULL;
	err = _mpi_req_build(buf, count, dtype, src, tag, comm,
				LAM_RQIRECV, req);

	if (err != MPI_SUCCESS) return(lam_errfunc(comm, BLKMPIRINIT, err));

	(*req)->rq_marks |= LAM_RQFPERSIST;

	lam_resetfunc_m(BLKMPIRINIT);
	return(MPI_SUCCESS);
}
