/*
 *	Ohio Trollius
 *	Copyright 1997 The Ohio State University
 *	RBD
 *
 *	$Id: sendrecv.c,v 6.2 1998/05/26 05:02:15 lam Exp $
 *
 *	Function:	- send and receive
 *	Accepts:	- send buffer
 *			- send count
 *			- send datatype
 *			- destination
 *			- send tag
 *			- receive buffer
 *			- receive count
 *			- receive datatype
 *			- source
 *			- receive tag
 *			- communicator
 *			- status (out)
 *	Returns: 	- MPI_SUCCESS or error code
 */

#include <blktype.h>
#include <mpi.h>
#include <mpisys.h>

int
MPI_Sendrecv(sbuf, scount, sdtype, dest, stag,
		rbuf, rcount, rdtype, src, rtag, comm, status)

void			*sbuf;
int			scount;
MPI_Datatype		sdtype;
int			dest;
int			stag;
void			*rbuf;
int			rcount;
MPI_Datatype		rdtype;
int			src;
int			rtag;
MPI_Comm		comm;
MPI_Status		*status;

{
	MPI_Request	req;			/* request */
	int		err;			/* error code */
	int		fl_trace;		/* do tracing? */

	lam_initerr_m();
	lam_setfunc_m(BLKMPISENDRECV);
/*
 * Generate a start wrapper trace.
 */
	fl_trace = ((_kio.ki_rtf & RTF_TRON) == RTF_TRON);

	if (fl_trace) {
		lam_tr_wrapstart(BLKMPISENDRECV);
	}
/*
 * Send & receive.  The arguments will be checked in the send/recv calls.
 */
	err = MPI_Irecv(rbuf, rcount, rdtype, src, rtag, comm, &req);
	if (err != MPI_SUCCESS) {
		return(lam_errfunc(comm, BLKMPISENDRECV, err));
	}

	err = MPI_Send(sbuf, scount, sdtype, dest, stag, comm);
	if (err != MPI_SUCCESS) {
		return(lam_errfunc(comm, BLKMPISENDRECV, err));
	}

	err = MPI_Wait(&req, status);
	if (err != MPI_SUCCESS) {
		return(lam_errfunc(comm, BLKMPISENDRECV, err));
	}
/*
 * Generate an end wrapper trace.
 */
	if (fl_trace) lam_tr_wrapend(BLKMPISENDRECV);

	lam_resetfunc_m(BLKMPISENDRECV);
	return(MPI_SUCCESS);
}
