/*
 *	Ohio Trollius
 *	Copyright 1997 The Ohio State University
 *	RBD
 *
 *	$Id: ssendinit.c,v 6.2 1998/05/26 05:02:15 lam Exp $
 *
 *	Function:	- create a synchronous send request
 *	Accepts:	- buffer
 *			- count
 *			- datatype
 *			- destination
 *			- tag
 *			- communicator
 *			- request (out)
 *	Returns: 	- MPI_SUCCESS or error code
 */

#include <blktype.h>
#include <mpi.h>
#include <mpisys.h>
#include <rpisys.h>


int
MPI_Ssend_init(buf, count, dtype, dest, tag, comm, req)

void			*buf;
int			count;
MPI_Datatype		dtype;
int			dest;
int			tag;
MPI_Comm		comm;
MPI_Request		*req;

{
	int		err;			/* error code */

	lam_initerr_m();
	lam_setfunc_m(BLKMPISSINIT);

	if (tag < 0 || tag > LAM_MAXTAG) {
		return(lam_errfunc(comm, BLKMPISSINIT,
				lam_mkerr(MPI_ERR_TAG, 0)));
	}

	if (dest == MPI_ANY_SOURCE) {
		return(lam_errfunc(comm, BLKMPISSINIT,
				lam_mkerr(MPI_ERR_RANK, 0)));
	}

	*req = MPI_REQUEST_NULL;
	err = _mpi_req_build(buf, count, dtype, dest, tag, comm,
				LAM_RQISSEND, req);

	if (err != MPI_SUCCESS) {
		return(lam_errfunc(comm, BLKMPISSINIT, err));
	}

	(*req)->rq_marks |= LAM_RQFPERSIST;

	lam_resetfunc_m(BLKMPISSINIT);
	return(MPI_SUCCESS);
}
