/*
 *	Ohio Trollius
 *	Copyright 1997 The Ohio State University
 *	NJN
 *
 *	$Id: tdelattr.c,v 0.2 1998/03/16 00:49:14 lam Exp $
 *
 *	Function:	- delete attribute
 *			- detach attribute from datatype
 *	Accepts:	- datatype
 *			- attribute key
 *	Returns:	- MPI_SUCCESS or error code
 */

#include <errno.h>

#include <blktype.h>
#include <mpi.h>
#include <mpisys.h>


int
MPI_Type_delete_attr(type, key)

MPI_Datatype		type;
int			key;

{
	struct _attrkey	*p;
	struct _attr	*pk;
	int		err;

	lam_initerr_m();
	lam_setfunc_m(BLKMPITDELETEATTR);
/*
 * Check the arguments.
 */
	if (type == MPI_DATATYPE_NULL) {
		return(lam_errfunc(MPI_COMM_WORLD,
			BLKMPITDELETEATTR, lam_mkerr(MPI_ERR_TYPE, 0)));
	}

	p = lam_getattr(key);
	if (p == 0) {
		return(lam_errfunc(MPI_COMM_WORLD,
			BLKMPITDELETEATTR, lam_mkerr(MPI_ERR_KEYVAL, 0)));
	}
/*
 * Check if the key is attached to the datatype.
 */
	pk = lam_getkey(type->dt_keys, key);
	if (pk == 0) {
		return(lam_errfunc(MPI_COMM_WORLD,
			BLKMPITDELETEATTR, lam_mkerr(MPI_ERR_KEYVAL, 0)));
	}
/*
 * Delete the attribute via the callback function.
 */
	if (((MPI_Type_delete_attr_function *) (p->ak_del))
			!= MPI_TYPE_NULL_DELETE_FN) {

		if (p->ak_flags & LAM_LANGF77) {
			(*((MPI_F_delete_function *) (p->ak_del)))
				(&type->dt_f77handle, &key,
				(int *) &(pk->a_value),
                               	(int *) &(p->ak_extra), &err);
		} else {
			err = (*((MPI_Type_delete_attr_function *) (p->ak_del)))
				(type, key, pk->a_value, p->ak_extra);
		}

		if (err != MPI_SUCCESS) {
			return(lam_err_comm(MPI_COMM_WORLD, err, 0,
						"deleting attribute"));
		}
	}
/*
 * Detach the key from the datatype.
 */
	if (lam_delkey(type->dt_keys, key)) {
		return(lam_err_comm(MPI_COMM_WORLD, MPI_ERR_INTERN, errno,
					"deleting key"));
	}

	lam_resetfunc_m(BLKMPITDELETEATTR);
	return(MPI_SUCCESS);
}
