/*
 *	Ohio Trollius
 *	Copyright 1997 The Ohio State University
 *	RBD/GDB/NJN
 *
 *	$Id: testall.c,v 6.2 1998/05/26 05:02:15 lam Exp $
 *
 *	Function:	- check if all requests completed
 *	Accepts:	- # of requests
 *			- array of requests
 *			- flag (out)
 *			- array of status structures (modified)
 *	Returns:	- MPI_SUCCESS or error code
 */

#include <blktype.h>
#include <mpi.h>
#include <mpisys.h>


int
MPI_Testall(count, reqs, flag, stats)

int			count;
MPI_Request		*reqs;
int			*flag;
MPI_Status		*stats;

{
	int		i;			/* favourite index */
	int		fl_done;		/* request done? */
	int		err;			/* error code */
	int		fl_trace;		/* do tracing? */
	MPI_Request	*req;			/* ptr request */
	MPI_Status	*stat;			/* ptr status */

	lam_initerr_m();
	lam_setfunc_m(BLKMPITESTALL);

	if (count < 0) {
		return(lam_errfunc(MPI_COMM_WORLD, BLKMPITESTALL,
				lam_mkerr(MPI_ERR_COUNT, 0)));
	}

	if ((count > 0) && (reqs == 0)) {
		return(lam_errfunc(MPI_COMM_WORLD, BLKMPITESTALL,
				lam_mkerr(MPI_ERR_ARG, 0)));
	}

	if (flag == 0) {
		return(lam_errfunc(MPI_COMM_WORLD, BLKMPITESTALL,
				lam_mkerr(MPI_ERR_ARG, 0)));
	}
/*
 * Generate a start wrapper trace.
 */
	fl_trace = ((_kio.ki_rtf & RTF_TRON) == RTF_TRON);

	if (fl_trace) {
		lam_tr_wrapstart(BLKMPITESTALL);
	}
/*
 * Initialize all status members.
 */
	if (stats != MPI_STATUSES_IGNORE) {
		for (i = 0, stat = stats; i < count; ++i, ++stat) {
			lam_emptystat(stat);
		}
	}
/*
 * Loop testing all requests non-destructively.
 * Stop at the first request not done.
 */
	req = reqs;
	stat = stats;

	for (i = 0; i < count; ++i, ++req) {

		err = lam_test(req, 0, &fl_done, stat);

		if (err != MPI_SUCCESS) {
			stat->MPI_ERROR = err;

			if (fl_trace) {
				lam_tr_wrapend(BLKMPITESTALL);
			}

			return(lam_errfunc(MPI_COMM_WORLD, BLKMPITESTALL,
					lam_mkerr(MPI_ERR_IN_STATUS, 0)));
		}

		if (fl_done == 0) {
			*flag = 0;

			if (fl_trace) {
				lam_tr_wrapend(BLKMPITESTALL);
			}

			lam_resetfunc_m(BLKMPITESTALL);
			return(MPI_SUCCESS);
		}

		if (stats != MPI_STATUSES_IGNORE) {
			stat++;
		}
	}
/*
 * All active requests are done, loop doing the job.
 */
	req = reqs;
	stat = stats;

	for (i = 0; i < count; ++i, ++req) {
		err = lam_test(req, 1, &fl_done, stat);

		if (err != MPI_SUCCESS) {
			stat->MPI_ERROR = err;

			if (fl_trace) {
				lam_tr_wrapend(BLKMPITESTALL);
			}

			return(lam_errfunc(MPI_COMM_WORLD, BLKMPITESTALL,
					lam_mkerr(MPI_ERR_IN_STATUS, 0)));
		}

		if (stats != MPI_STATUSES_IGNORE) {
			stat++;
		}
	}

	*flag = 1;
/*
 * Generate an end wrapper trace.
 */
	if (fl_trace) {
		lam_tr_wrapend(BLKMPITESTALL);
	}

	lam_resetfunc_m(BLKMPITESTALL);
	return(MPI_SUCCESS);
}
