/*
 *	Ohio Trollius
 *	Copyright 1997 The Ohio State University
 *	NJN/RBD
 *
 *	$Id: tindex.c,v 6.2 1998/05/26 05:02:15 lam Exp $
 *
 *	Function:	- create indexed derived datatype
 *	Accepts:	- # blocks
 *			- block length array
 *			- block displacement array
 *			- element datatype
 *			- new datatype (out)
 *	Returns:	- MPI_SUCCESS or error code
 */

#include <stdlib.h>

#include <blktype.h>
#include <mpi.h>
#include <mpisys.h>
#include <terror.h>

int
MPI_Type_indexed(count, lengths, disps, oldtype, newtype)

int			count;
int			*lengths;
int			*disps;
MPI_Datatype		oldtype;
MPI_Datatype		*newtype;

{
	MPI_Datatype	ntype;			/* new datatype */
	int		oldext;			/* extent of oldtype */
	int		i;

	lam_initerr();
	lam_setfunc(BLKMPITINDEX);
/*
 * Check the arguments.
 */
	if (oldtype == MPI_DATATYPE_NULL) {
		return(lam_errfunc(MPI_COMM_WORLD,
			BLKMPITINDEX, lam_mkerr(MPI_ERR_TYPE, 0)));
	}

	if (count < 0) {
		return(lam_errfunc(MPI_COMM_WORLD,
			BLKMPITINDEX, lam_mkerr(MPI_ERR_COUNT, 0)));
	}

	if ((newtype == 0) ||
			((count > 0) && ((lengths == 0) || (disps == 0)))) {
		return(lam_errfunc(MPI_COMM_WORLD,
			BLKMPITINDEX, lam_mkerr(MPI_ERR_ARG, 0)));
	}
/*
 * Create the new datatype.
 */
	ntype = (MPI_Datatype) malloc(sizeof(struct _dtype));
	if (ntype == 0) {
		return(lam_errfunc(MPI_COMM_WORLD,
			BLKMPITINDEX, lam_mkerr(MPI_ERR_OTHER, errno)));
	}

	lam_type_setdefaults(ntype);

	if (count > 0) {
		ntype->dt_disps = (MPI_Aint *) malloc((unsigned) (count *
					(sizeof(MPI_Aint) + sizeof(int))));

		if (ntype->dt_disps == 0) {
			free((char *) ntype);
			return(lam_errfunc(MPI_COMM_WORLD, BLKMPITINDEX,
					lam_mkerr(MPI_ERR_OTHER, errno)));
		}

		ntype->dt_lengths = (int *) (((char *) ntype->dt_disps) +
						(count * sizeof(MPI_Aint)));
	}

	for (i = 0; i < count; ++i) {
		ntype->dt_disps[i] = disps[i];

		if ((ntype->dt_lengths[i] = lengths[i]) < 0) {
			free((char *) ntype->dt_disps);
			free((char *) ntype);
			return(lam_errfunc(MPI_COMM_WORLD,
				BLKMPITINDEX, lam_mkerr(MPI_ERR_ARG, 0)));
		}
	}

	ntype->dt_format = LAM_DTINDEXED;
	ntype->dt_count = count;
	ntype->dt_dtype = oldtype;
	oldtype->dt_refcount++;

	if (count > 0) {
		oldext = oldtype->dt_upper - oldtype->dt_lower;
		
		for (i = 0; i < count; ++i) {
			lam_dtblock(ntype, oldtype, lengths[i],
					disps[i] * oldext);
		}

		if (ntype->dt_size > 0 && ntype->dt_lower != 0) {
			ntype->dt_flags &= ~LAM_DTNOPACK;
		}

		lam_dtalign(ntype);
	}
	else {
		ntype->dt_flags = LAM_DTNOPACK | LAM_DTNOXADJ;
		ntype->dt_size = 0;
	}

	*newtype = ntype;

	lam_resetfunc(BLKMPITINDEX);
	return(MPI_SUCCESS);
}
