/*
 *	Ohio Trollius
 *	Copyright 1997 The Ohio State University
 *	JGH
 *
 *	$Id: tsetattr.c,v 0.1 1997/11/23 12:52:14 lam Exp $
 *
 *	Function:	- put value in attribute
 *			- assign attribute to datatype
 *	Accepts:	- datatype
 *			- attribute key
 *			- attribute value
 *	Returns:	- MPI_SUCCESS or error code
 */

#include <errno.h>

#include <blktype.h>
#include <mpi.h>
#include <mpisys.h>


int
MPI_Type_set_attr(type, key, value)

MPI_Datatype		type;
int			key;
void			*value;

{
	struct _attrkey	*p;
	struct _attr	*pk;
	int		err;

	lam_initerr_m();
	lam_setfunc_m(BLKMPITSETATTR);
/*
 * Check the arguments.
 */
	if (type == MPI_DATATYPE_NULL) {
		return(lam_errfunc(MPI_COMM_WORLD,
			BLKMPITSETATTR, lam_mkerr(MPI_ERR_TYPE, 0)));
	}

	p = lam_getattr(key);
	if (p == 0 || (p->ak_flags & LAM_PREDEF)
			|| !(p->ak_flags & LAM_AKTYPE)) {
		return(lam_errfunc(MPI_COMM_WORLD,
			BLKMPITSETATTR, lam_mkerr(MPI_ERR_KEYVAL, 0)));
	}
/*
 * If the key is not attached to the datatype, attach and fill it.
 */
	pk = (struct _attr *) lam_getkey(type->dt_keys, key);
	if (pk == 0) {
		if (lam_putkey(&type->dt_keys, key)) {
			return(lam_errfunc(MPI_COMM_WORLD, BLKMPITSETATTR,
				lam_mkerr(MPI_ERR_OTHER, errno)));
		}

		pk = (struct _attr *) lam_getkey(type->dt_keys, key);
	}
/*
 * Old attribute, delete previous value and fill new one.
 */
	else if (((MPI_Type_delete_attr_function *) (p->ak_del))
			!= MPI_TYPE_NULL_DELETE_FN) {

		if (p->ak_flags & LAM_LANGF77) {
			(*((MPI_F_delete_function *) (p->ak_del)))
				(&type->dt_f77handle, &key,
				(int *) &pk->a_value,
				(int *) &p->ak_extra, &err);
		} else {
			err = (*((MPI_Type_delete_attr_function *) (p->ak_del)))
				(type, key, pk->a_value, p->ak_extra);
		}

		if (err != MPI_SUCCESS) {
			return(lam_err_comm(MPI_COMM_WORLD, err, 0,
						"deleting old attribute"));
		}
	}

	pk->a_value = value;

	lam_resetfunc_m(BLKMPITSETATTR);
	return(MPI_SUCCESS);
}
