/*
 *	Ohio Trollius
 *	Copyright 1997 The Ohio State University
 *	RBD
 *
 *	$Id: unpack.c,v 6.2 1998/05/26 05:02:15 lam Exp $
 *
 *	Function:	- unpack to data buffer
 *	Accepts:	- packed buffer
 *			- packed buffer size
 *			- packed buffer position (in-out)
 *			- data buffer
 *			- data count
 *			- data type
 *			- communicator
 *	Returns:	- MPI_SUCCESS or error code
 */

#include <blktype.h>
#include <mpi.h>
#include <mpisys.h>

int
MPI_Unpack(packbuf, packsize, packpos, buf, count, dtype, comm)

void			*packbuf;
int			packsize;
int			*packpos;
void			*buf;
int			count;
MPI_Datatype		dtype;
MPI_Comm		comm;

{
	int		size;
	char		*p;

	lam_initerr();
	lam_setfunc(BLKMPIUNPACK);
/*
 * Check the arguments.
 */
	if (comm == MPI_COMM_NULL) {
		return(lam_errfunc(MPI_COMM_WORLD,
			BLKMPIUNPACK, lam_mkerr(MPI_ERR_COMM, 0)));
	}

	if (dtype == MPI_DATATYPE_NULL || (!dtype->dt_commit)) {
		return(lam_errfunc(comm,
			BLKMPIUNPACK, lam_mkerr(MPI_ERR_TYPE, 0)));
	}

	if (count < 0) {
		return(lam_errfunc(comm,
			BLKMPIUNPACK, lam_mkerr(MPI_ERR_COUNT, 0)));
	}

	if ((packsize < 0) || (packpos == 0)) {
		return(lam_errfunc(comm,
			BLKMPIUNPACK, lam_mkerr(MPI_ERR_ARG, 0)));
	}

	if ((*packpos < 0) || ((*packpos >= packsize) && (count > 0))) {
		return(lam_errfunc(comm,
			BLKMPIUNPACK, lam_mkerr(MPI_ERR_ARG, 0)));
	}

	if ((packsize > 0) && (packbuf == 0)) {
		return(lam_errfunc(comm,
			BLKMPIUNPACK, lam_mkerr(MPI_ERR_BUFFER, 0)));
	}
/*
 * Unpack the data.
 */
	p = ((char *) packbuf) + *packpos;
	size = packsize - *packpos;

	size = lam_unpack(p, size, (char *) buf, count, dtype);

	if (size < 0) {
		return(lam_errfunc(comm,
			BLKMPIUNPACK, lam_mkerr(MPI_ERR_TRUNCATE, 0)));
	}

	*packpos += size;

	lam_resetfunc(BLKMPIUNPACK);
	return(MPI_SUCCESS);
}
