/*
 *	Ohio Trollius
 *	Copyright 1997 The Ohio State University
 *	NJN
 *
 *	$Id: wcomplete.c,v 0.1 1997/11/23 12:54:15 lam Exp $
 *
 *	Function:	- complete a window
 *	Accepts:	- window
 *	Returns:	- MPI_SUCCESS or error code
 */

#include <errno.h>
#include <stdlib.h>

#include <blktype.h>
#include <mpi.h>
#include <mpisys.h>
#include <rpisys.h>


int
MPI_Win_complete(win)

MPI_Win			win;

{
	MPI_Comm	comm;			/* window's communicator */
	MPI_Request	*reqs;			/* completing message reqs */
	MPI_Request	*r;
	int		complete_msg[1];	/* complete message */
	int		err;			/* error code */
	int		i;

	lam_initerr_m();
	lam_setfunc_m(BLKMPIWINCOMPLETE);
/*
 * Check arguments.
 */
	if (win == MPI_WIN_NULL) {
		return(lam_errfunc(MPI_COMM_WORLD,
			BLKMPIWINCOMPLETE, lam_mkerr(MPI_ERR_WIN, 0)));
	}

	comm = win->w_comm;

	if (!(win->w_flags & LAM_WFSTARTED)) {
		return(lam_errfunc(comm,
			BLKMPIWINCOMPLETE, lam_mkerr(MPI_ERR_EPOCH, 0)));
	}
/*
 * Send "i-am-completing" mesages to all target processes in the 2 party
 * synchronization.  
 */
	reqs = (MPI_Request *) malloc(win->w_ngroup * sizeof(MPI_Request));
	if (reqs == 0) {
		return(lam_errfunc(comm,
			BLKMPIWINCOMPLETE, lam_mkerr(MPI_ERR_OTHER, errno)));
	}

	complete_msg[0] = LAM_OSD_COMPLETE;

	for (i = 0, r = reqs; i < comm->c_group->g_nprocs; i++) {

		if (win->w_pstate[i] & LAM_WFSTARTED) {
			err = MPI_Send_init(complete_msg, 1, MPI_INT, i,
						LAM_OSD_HEADER, comm, r++);

			if (err != MPI_SUCCESS) {
				return(lam_errfunc(comm,
						BLKMPIWINCOMPLETE, err));
			}

			win->w_pstate[i] &= ~LAM_WFSTARTED;
		}
	}

	err = MPI_Startall(win->w_ngroup, reqs);
	if (err != MPI_SUCCESS) {
		return(lam_errfunc(win->w_comm, BLKMPIWINCOMPLETE, err));
	}
/*
 * Complete all locally originated one-sided requests in the window.
 */
	err = lam_osd_complete(win, LAM_RQFOSORIG);
	if (err != MPI_SUCCESS) {
		return(lam_errfunc(comm, BLKMPIWINCOMPLETE, err));
	}
/*
 * Complete the "i-am-completing" messages.
 */
	err = MPI_Waitall(win->w_ngroup, reqs, MPI_STATUSES_IGNORE);
	if (err != MPI_SUCCESS) {
		return(lam_errfunc(comm, BLKMPIWINCOMPLETE, err));
	}
	free((char *) reqs);

	win->w_ngroup = 0;
	win->w_flags &= ~LAM_WFSTARTED;

        lam_resetfunc_m(BLKMPIWINCOMPLETE);
	return(MPI_SUCCESS);
}
