/*
 *	Ohio Trollius
 *	Copyright 1997 The Ohio State University
 *	NJN
 *
 *	$Id: wdelattr.c,v 0.2 1998/03/16 00:50:06 lam Exp $
 *
 *	Function:	- delete attribute
 *			- detach attribute from window
 *	Accepts:	- window
 *			- attribute key
 *	Returns:	- MPI_SUCCESS or error code
 */

#include <errno.h>

#include <blktype.h>
#include <mpi.h>
#include <mpisys.h>


int
MPI_Win_delete_attr(win, key)

MPI_Win			win;
int			key;

{
	struct _attrkey	*p;
	struct _attr	*pk;
	int		err;

	lam_initerr_m();
	lam_setfunc_m(BLKMPIWINDELETEATTR);
/*
 * Check the arguments.
 */
	if (win == MPI_WIN_NULL) {
		return(lam_errfunc(MPI_COMM_WORLD,
			BLKMPIWINDELETEATTR, lam_mkerr(MPI_ERR_WIN, 0)));
	}

	p = lam_getattr(key);
	if (p == 0) {
		return(lam_errfunc(win->w_comm,
			BLKMPIWINDELETEATTR, lam_mkerr(MPI_ERR_KEYVAL, 0)));
	}
/*
 * Check if the key is attached to the window.
 */
	pk = lam_getkey(win->w_keys, key);
	if (pk == 0) {
		return(lam_errfunc(win->w_comm,
			BLKMPIWINDELETEATTR, lam_mkerr(MPI_ERR_KEYVAL, 0)));
	}
/*
 * Delete the attribute via the callback function.
 */
	if (((MPI_Win_delete_attr_function *) (p->ak_del))
			!= MPI_WIN_NULL_DELETE_FN) {

		if (p->ak_flags & LAM_LANGF77) {
			(*((MPI_F_delete_function *)(p->ak_del)))
				(&win->w_f77handle, &key,
				(int *) &(pk->a_value),
				(int *) &(p->ak_extra), &err);
		} else {
			err = (*((MPI_Win_delete_attr_function *) (p->ak_del)))
				(win, key, pk->a_value, p->ak_extra);
		}

		if (err != MPI_SUCCESS) {
			return(lam_err_win(win, err, 0, "deleting attribute"));
		}
	}
/*
 * Detach the key from the window.
 */
	if (lam_delkey(win->w_keys, key)) {
		return(lam_err_win(win, MPI_ERR_INTERN, errno, "deleting key"));
	}

	lam_resetfunc_m(BLKMPIWINDELETEATTR);
	return(MPI_SUCCESS);
}
