/*
 *	Ohio Trollius
 *	Copyright 1997 The Ohio State University
 *	NJN
 *
 *	$Id: wfree.c,v 0.2 1998/03/16 00:50:51 lam Exp $
 *
 *	Function:	- free window
 *	Accepts:	- window (out)
 *	Returns:	- MPI_SUCCESS or error code
 */

#include <stdlib.h>

#include <blktype.h>
#include <mpi.h>
#include <mpisys.h>
#include <rpisys.h>


int
MPI_Win_free(win)

MPI_Win			*win;

{
	MPI_Win		w;			/* window being freed */
	struct _attrkey *keystate;		/* key state */
	struct _attr	*p;			/* attribute */
	int		key;
	int		err;

	lam_initerr_m();
	lam_setfunc_m(BLKMPIWINFREE);

	if (win == 0 || *win == MPI_WIN_NULL) {
		return(lam_errfunc(MPI_COMM_WORLD,
			BLKMPIWINFREE, lam_mkerr(MPI_ERR_WIN, 0)));
	}

	w = *win;
/*
 * Make sure all processes are participating.
 */
	err = MPI_Barrier(w->w_comm);
	if (err != MPI_SUCCESS) {
	    return(lam_errfunc(w->w_comm, BLKMPIWINFREE, err));
	}
/*
 * Destroy the background request.
 */
	if (w->w_req->rq_state == LAM_RQSACTIVE) {
		return(lam_errfunc(w->w_comm, BLKMPIWINFREE,
				lam_mkerr(MPI_ERR_EPOCH, 0)));
	} else {
		_mpi_req_rem_m(w->w_req);
		err = _mpi_req_destroy(&w->w_req);
		if (err != MPI_SUCCESS) {
			return(lam_errfunc(w->w_comm, BLKMPIWINFREE, err));
		}
	}
/*
 * Free attributes and keys which are not pre-defined.
 */
	if (w->w_keys) {
		p = (struct _attr *) ah_top(w->w_keys);
		while (p) {
			key = p->a_key;
			keystate = lam_getattr(key);

			err = MPI_Win_delete_attr(w, key);
			if (err != MPI_SUCCESS) {
				return(lam_errfunc(w->w_comm,
						BLKMPIWINFREE, err));
			}

			if (!(keystate->ak_flags & LAM_PREDEF)) {
				err = MPI_Win_free_keyval(&key);
				if (err != MPI_SUCCESS) {
					return(lam_errfunc(w->w_comm,
							BLKMPIWINFREE, err));
				}
			}

			p = (struct _attr *) ah_next(w->w_keys, (void *) p);
		}
		ah_free(w->w_keys);
	}
/*
 * Free the underlying communicator.
 */
	err = MPI_Comm_free(&w->w_comm);
	if (err != MPI_SUCCESS) {
		return(lam_errfunc(MPI_COMM_WORLD, BLKMPIWINFREE, err));
	}

	free((char *) w);
	*win = MPI_WIN_NULL;

        lam_resetfunc_m(BLKMPIWINFREE);
	return(MPI_SUCCESS);
}
