/*
 *	Ohio Trollius
 *	Copyright 1997 The Ohio State University
 *	NJN
 *
 *	$Id: wpost.c,v 0.1 1997/11/23 12:54:56 lam Exp $
 *
 *	Function:	- post a window
 *	Accepts:	- group
 *			- assertion
 *			- window
 *	Returns:	- MPI_SUCCESS or error code
 */

#include <errno.h>
#include <stdlib.h>

#include <blktype.h>
#include <mpi.h>
#include <mpisys.h>


int
MPI_Win_post(group, assertion, win)

MPI_Group		group;
int			assertion;
MPI_Win			win;

{
	int		rank;			/* rank in window */
	int		err;
	int		i;

	lam_initerr_m();
	lam_setfunc_m(BLKMPIWINPOST);
/*
 * Check arguments.
 */
	if (win == MPI_WIN_NULL) {
		return(lam_errfunc(MPI_COMM_WORLD,
			BLKMPIWINPOST, lam_mkerr(MPI_ERR_WIN, 0)));
	}

	if (group == MPI_GROUP_NULL || group == MPI_GROUP_EMPTY) {
		return(lam_errfunc(win->w_comm,
			BLKMPIWINPOST, lam_mkerr(MPI_ERR_GROUP, 0)));

	}
/*
 * Check epoch.
 */
	if (win->w_flags & (LAM_WFEXPEPOCH | LAM_WFPOSTED)) {
		return(lam_errfunc(win->w_comm,
			BLKMPIWINPOST, lam_mkerr(MPI_ERR_EPOCH, 0)));
	}
/*
 * Send "i-post-u" messages.
 */
	win->w_flags |= LAM_WFPOSTED | LAM_WFNOOUT;
	win->w_ncomplete = group->g_nprocs;

	for (i = 0; i < group->g_nprocs; i++) {
		rank = lam_rank_in_group(win->w_comm->c_group,
						group->g_procs[i]);

		if (rank == MPI_UNDEFINED) {
			return(lam_errfunc(win->w_comm,
				BLKMPIWINPOST, lam_mkerr(MPI_ERR_GROUP, 0)));
		}

		win->w_pstate[rank] |= LAM_WFPOSTED;

		err = lam_osd_send(0, 0, MPI_BYTE, rank,
				LAM_OSD_POST, win->w_comm, TRUE, FALSE);

		if (err != MPI_SUCCESS) {
			return(lam_errfunc(win->w_comm, BLKMPIWINPOST, err));
		}
	}

        lam_resetfunc_m(BLKMPIWINPOST);
	return(MPI_SUCCESS);
}
