/*
 *	Ohio Trollius
 *	Copyright 1997 The Ohio State University
 *	NJN
 *
 *	$Id: wseterr.c,v 0.2 1998/03/16 00:51:04 lam Exp $
 *
 *	Function:	- set error handler for window
 *	Accepts:	- communicator
 *			- error handler
 *	Returns:	- MPI_SUCCESS or error code
 */

#include <blktype.h>
#include <mpi.h>
#include <mpisys.h>

int
MPI_Win_set_errhandler(win, errhdl)

MPI_Win			win;
MPI_Errhandler		errhdl;

{
	int		err;

	lam_initerr_m();
	lam_setfunc_m(BLKMPIWINSETERRHDL);
/*
 * Check the arguments.
 */
	if (win == MPI_WIN_NULL) {
		return(lam_errfunc(MPI_COMM_WORLD,
			BLKMPIWINSETERRHDL, lam_mkerr(MPI_ERR_WIN, 0)));
	}

	if (errhdl == MPI_ERRHANDLER_NULL || !(errhdl->eh_flags & LAM_EHWIN)) {
		return(lam_errfunc(win->w_comm,
			BLKMPIWINSETERRHDL, lam_mkerr(MPI_ERR_ARG, 0)));
	}
/*
 * Free previous error handler.
 */
	err = MPI_Errhandler_free(&(win->w_errhdl));
	if (err != MPI_SUCCESS) {
		return(lam_errfunc(win->w_comm, BLKMPIWINSETERRHDL, err));
	}
/*
 * Set the communicator's new error handler.
 */
	win->w_errhdl = errhdl;
	errhdl->eh_refcount++;

	lam_resetfunc_m(BLKMPIWINSETERRHDL);
	return(MPI_SUCCESS);
}
