/*
 *	Ohio Trollius
 *	Copyright 1997 The Ohio State University
 *	NJN
 *
 *	$Id: wstart.c,v 0.1 1997/11/23 12:54:56 lam Exp $
 *
 *	Function:	- start a window
 *	Accepts:	- group
 *			- assertion
 *			- window
 *	Returns:	- MPI_SUCCESS or error code
 */

#include <errno.h>
#include <stdlib.h>

#include <blktype.h>
#include <mpi.h>
#include <mpisys.h>
#include <rpisys.h>


int
MPI_Win_start(group, assertion, win)

MPI_Group		group;
int			assertion;
MPI_Win			win;

{
	MPI_Request	*reqs;
	int		rank;			/* rank in window */
	int		err;			/* error code */
	int		i;

	lam_initerr_m();
	lam_setfunc_m(BLKMPIWINSTART);
/*
 * Check arguments.
 */
	if (win == MPI_WIN_NULL) {
		return(lam_errfunc(MPI_COMM_WORLD,
			BLKMPIWINSTART, lam_mkerr(MPI_ERR_WIN, 0)));
	}

	if (group == MPI_GROUP_NULL || group == MPI_GROUP_EMPTY) {
		return(lam_errfunc(win->w_comm,
			BLKMPIWINSTART, lam_mkerr(MPI_ERR_GROUP, 0)));
	}
/*
 * Check epoch.
 */
	if (win->w_flags & (LAM_WFACCEPOCH | LAM_WFSTARTED)) {
		return(lam_errfunc(win->w_comm,
			BLKMPIWINSTART, lam_mkerr(MPI_ERR_EPOCH, 0)));
	}
/*
 * Wait to be posted by the group.
 */
	win->w_flags |= LAM_WFSTARTED | LAM_WFNOOUT;
	win->w_ngroup = group->g_nprocs;

	reqs = (MPI_Request *) malloc(group->g_nprocs * sizeof(MPI_Request));
	if (reqs == 0) {
		return(lam_errfunc(win->w_comm,
			BLKMPIWINSTART, lam_mkerr(MPI_ERR_OTHER, errno)));
	}

	for (i = 0; i < group->g_nprocs; i++) {
		rank = lam_rank_in_group(win->w_comm->c_group,
						group->g_procs[i]);

		if (rank == MPI_UNDEFINED) {
			return(lam_errfunc(win->w_comm,
				BLKMPIWINSTART, lam_mkerr(MPI_ERR_GROUP, 0)));
		}

		win->w_pstate[rank] |= LAM_WFSTARTED;

		err = MPI_Recv_init(0, 0, MPI_BYTE, rank,
				LAM_OSD_POST, win->w_comm, reqs + i);

		if (err != MPI_SUCCESS) {
			return(lam_errfunc(win->w_comm, BLKMPIWINSTART, err));
		}
	}

	err = MPI_Startall(group->g_nprocs, reqs);
	if (err != MPI_SUCCESS) {
		return(lam_errfunc(win->w_comm, BLKMPIWINSTART, err));
	}

	err = MPI_Waitall(group->g_nprocs, reqs, MPI_STATUSES_IGNORE);
	if (err != MPI_SUCCESS) {
		return(lam_errfunc(win->w_comm, BLKMPIWINSTART, err));
	}

	free((char *) reqs);

        lam_resetfunc_m(BLKMPIWINSTART);
	return(MPI_SUCCESS);
}
