/*
 *	Ohio Trollius
 *	Copyright 1997 The Ohio State University
 *	NJN
 *
 *	$Id: wwait.c,v 0.1 1997/11/23 12:55:29 lam Exp $
 *
 *	Function:	- one sided window wait
 *	Accepts:	- window
 *	Returns:	- MPI_SUCCESS or error code
 */

#include <blktype.h>
#include <mpi.h>
#include <mpisys.h>
#include <rpisys.h>


int
MPI_Win_wait(win)

MPI_Win			win;

{
	MPI_Comm	comm;
	int		rank;			/* rank in window */
	int		err;			/* error code */

	lam_initerr_m();
	lam_setfunc_m(BLKMPIWINWAIT);
/*
 * Check arguments.
 */
	if (win == MPI_WIN_NULL) {
		return(lam_errfunc(MPI_COMM_WORLD,
			BLKMPIWINWAIT, lam_mkerr(MPI_ERR_WIN, 0)));
	}

	comm = win->w_comm;
	rank = comm->c_group->g_myrank;

	if (!(win->w_flags & LAM_WFPOSTED)) {
		return(lam_errfunc(comm,
			BLKMPIWINWAIT, lam_mkerr(MPI_ERR_EPOCH, 0)));
	}
/*
 * Loop until everybody has completed.
 */
	_mpi_req_blkclr_m();
	_mpi_req_blkset_m(win->w_req);

	while (win->w_ncomplete > 0) {

		err = lam_osd_blkadv(win);
		if (err != MPI_SUCCESS) {
			return(lam_errfunc(comm, BLKMPIWINWAIT, err));
		}
	}
/*
 * Complete all remote one-sided requests in the window.
 */
	err = lam_osd_complete(win, LAM_RQFOSTARG);
	if (err != MPI_SUCCESS) {
		return(lam_errfunc(comm, BLKMPIWINWAIT, err));
	}

	win->w_flags &= ~LAM_WFPOSTED;

        lam_resetfunc_m(BLKMPIWINWAIT);
	return(MPI_SUCCESS);
}
