/*
 *	Ohio Trollius
 *	Copyright 1996 The Ohio State University
 *	GDB
 *
 *	$Id: rtrget.c,v 6.1 1996/11/24 00:26:17 nevin Rel $
 *
 *	Function:	- unloads trace data from remote trace daemon
 */

#include <stdlib.h>
#include <unistd.h>

#include <events.h>
#include <ksignal.h>
#include <net.h>
#include <trreq.h>
#include <typical.h>

/*
 * external variables
 */
extern int		errno;

/*
 * local functions
 */
static int4		rtr();

/*
 *	lam_rtrget
 *
 *	Function:	- reads trace data from remote node
 *	Accepts:	- target node ID
 *			- list number
 *			- process ID
 *			- buffer ptr ptr
 *	Returns:	- total length of received trace data
 */
int4
lam_rtrget(nodeid, lnum, pid, buf)

int4			nodeid;
int4			lnum;
int4			pid;
char			**buf;

{
	return(rtr(nodeid, lnum, pid, buf, TRQGET));
}

/*
 *	lam_rtrforget
 *
 *	Function:	- destructively reads trace data from remote node
 *	Accepts:	- target node ID
 *			- list number
 *			- process ID
 *			- buffer ptr ptr
 *	Returns:	- total length of received trace data
 */
int4
lam_rtrforget(nodeid, lnum, pid, buf)

int4			nodeid;
int4			lnum;
int4			pid;
char			**buf;

{
	return(rtr(nodeid, lnum, pid, buf, TRQFORGET));
}

/*
 *	rtr
 *
 *	Function:	- unloads trace data from remote node
 *			- writes trace data into malloc'd buffer
 *	Accepts:	- target node ID
 *			- list number
 *			- process ID
 *			- trace buffer ptr ptr (value returned)
 *			- traced request code
 *	Returns:	- total length of received trace data
 */
static int4
rtr(nodeid, lnum, pid, buf, req)

int4			nodeid;
int4			lnum;
int4			pid;
int4			req;
char			**buf;

{
	struct nmsg	nhead;		/* network message desc. */
	struct trreq  	*request;	/* traced request */
	struct trreply	*reply;		/* traced reply */
	int4		trtotal;	/* total matching traces */
	int4		trleft;		/* traces left to receive */
	int4		mask;		/* signal mask */

	request = (struct trreq *) nhead.nh_data;
	reply = (struct trreply *) nhead.nh_data;

	request->trq_src_node = (nodeid == LOCAL) ? nodeid : getnodeid();
	request->trq_src_event = -getpid();
	request->trq_req = req;
	request->trq_lnum = lnum;
	request->trq_pid = pid;

	nhead.nh_node = nodeid;
	nhead.nh_event = EVTRACED;
	nhead.nh_type = 0;
	nhead.nh_flags = 0;
	nhead.nh_length = 0;
	nhead.nh_msg = 0;

	mask = ksigblock(sigmask(SIGUDIE) | sigmask(SIGARREST));

	if (nsend(&nhead)) {
		ksigsetmask(mask);
		return(ERROR);
	}
/*
 * Receive total trace length.
 */
	nhead.nh_event = -getpid();
	nhead.nh_type = 0;
	nhead.nh_length = 0;
	nhead.nh_msg = 0;

	if (nrecv(&nhead)) {
		ksigsetmask(mask);
		return(ERROR);
	}

	trtotal = reply->trr_length;
	trleft = reply->trr_length;

	*buf = malloc((unsigned) trtotal);

	if (*buf == 0) {
		ksigsetmask(mask);
		return(ERROR);
	}

	nhead.nh_msg = *buf;

	while ((trleft > 0) && (reply->trr_errno == 0)) {
		nhead.nh_type = 0;
		nhead.nh_length = MAXNMSGLEN;

		if (nrecv(&nhead)) {
			free(*buf);
			ksigsetmask(mask);
			return(ERROR);
		}

		trleft -= nhead.nh_length;
		nhead.nh_msg += nhead.nh_length;
	}

	ksigsetmask(mask);

	if (reply->trr_errno) {
		errno = reply->trr_errno;
		free(*buf);
		return(ERROR);
	} else {
		return(trtotal);
	}
}
