/*
 *	Ohio Trollius
 *	Copyright 1997 The Ohio State University
 *	SBD/JRV
 *
 *	$Id: tfprint.c,v 6.2 1998/05/26 04:59:30 lam Exp $
 *
 *	Function:	- prints on stdout
 *	Accepts:	- format string
 *			- variable number of items to print
 *	Returns:	- the number of printed characters, or EOF if error
 */

#if __STDC__
#include <stdarg.h>
#else
#include <varargs.h>
#endif

#include <laminternal.h>
#include <tstdio.h>

#if __STDC__
int
tprintf(const char *format, ...)
#else
int
tprintf(format, va_alist)

char			*format;
va_dcl
#endif

{
	char		*storep	= 0;
	TFILE		*fp	= tstdout;
	int		count;
	va_list		arglist;

#if __STDC__
	va_start(arglist, format);
#else
	va_start(arglist);
#endif
	count = _fmt(fp, storep, format, arglist);
	va_end(arglist);
	return(count);
}


/*
 *	tfprintf
 *
 *	Function:	- prints to file pointed to by fp
 *	Accepts:	- file pointer
 *			- format string
 *			- variable number of items to print
 *	Returns:	- the number of printed characters, of EOF if error
 */
#if __STDC__
int
tfprintf(TFILE *fp, const char *format, ...)
#else
int
tfprintf(fp, format, va_alist)

TFILE			*fp;
char			*format;
va_dcl

#endif

{
	char		*storep	= 0;
	int		count;
	va_list		arglist;

#if __STDC__
	va_start(arglist, format);
#else
	va_start(arglist);
#endif
	count = _fmt(fp, storep, format, arglist);
	va_end(arglist);
	return(count);
}


/*
 *	tsprintf
 *
 *	Function:	- prints memory location pointed to by storep
 *	Accepts:	- char pointer to destination
 *			- format string
 *			- variable number of items to format
 *	Returns:	- char pointer to destination
 */	
#if __STDC__
char *
tsprintf(char *storep, const char *format, ...)
#else
char *
tsprintf(storep, format, va_alist)

char			*storep;
char			*format;
va_dcl

#endif

{
	TFILE		*fp = 0;
	int		count;
	va_list		arglist;

#if __STDC__
	va_start(arglist, format);
#else
	va_start(arglist);
#endif
	count = _fmt(fp, storep, format, arglist);
	va_end(arglist);
	
	return(storep + count);
}


/*
 *	tvprintf
 *
 *	Function:	- same as printf(), but this takes a va_list
 *	Accepts:	- format string
 *			- variable number of items to format in a va_list
 *	Returns:	- the number of printed characters, of EOF if error
 */
#if __STDC__
int
tvprintf(const char *format, ...)
#else
int
tvprintf(format, va_alist)

char			*format;
va_dcl
#endif

{
	char		*storep	= 0;
	TFILE		*fp	= tstdout;
	int		count;
	va_list		ap;

#if __STDC__
	va_start(ap, format);
#else
	va_start(ap);
#endif
	count = _fmt(fp, storep, format, ap);
	va_end(ap);
	return(count);
}


/*
 *      tvfprintf
 *
 *	Function:	- same as fprintf(), but this takes a va_list
 *	Accepts:	- file pointer
 *			- format string
 *			- variable number of items to format in a va_list
 *	Returns:	- the number of printed characters, of EOF if error
 */
#if __STDC__
int
tvfprintf(TFILE *fp, const char *format, ...)
#else
int
tvfprintf(fp, format, arglist)

TFILE			*fp;
char			*format;
va_list			arglist;
#endif
{
	char		*storep	= 0;
	int		count;
#if __STDC__
	va_list		arglist;

	va_start(arglist, format);
#endif
	count = _fmt(fp, storep, format, arglist);
#if __STDC__
	va_end(arglist);
#endif
	return(count);
}


/*
 *	tvsprintf
 *
 *	Function:	- same as sprintf, but takes a va_list
 *	Accepts:	- char pointer to destination
 *			- format string
 *			- variable number of items to format in a va_list
 *	Returns:	- char pointer to destination
 */
#if __STDC__
char *
tvsprintf(char *storep, const char *format, ...)
#else
char *
tvsprintf(storep, format, arglist)

char			*storep;
char			*format;
va_list			arglist;
#endif
{
	TFILE		*fp = 0;
	int		count;
#if __STDC__
	va_list		arglist;

	va_start(arglist, format);
#endif
	count = _fmt(fp, storep, format, arglist);
#if __STDC__
	va_end(arglist);
#endif
	return(storep + count);
}
