/*
 *	Ohio Trollius
 *	Copyright 1997 The Ohio State University
 *	GDB/RBD/NJN
 *
 *	$Id: hf77.c,v 6.4 1998/09/29 11:07:12 lamteam Exp $
 *
 *	Function:	- wrapper for FORTRAN program compilation
 */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>

#include <args.h>
#include <boot.h>
#include <portable.h>

#ifndef DEFTF77
#define DEFTF77 "f77"
#endif

#ifndef FLIBPATH
#define FLIBPATH "-L"
#endif

/*
 * external functions
 */
extern int		cnfexec();

/*
 * local variables
 */
static char		ph[LAM_PATH_MAX];
static char		plib[LAM_PATH_MAX];

int
main(argc, argv)

int			argc;
char			*argv[];

{
	int		ac;		/* f77 # args */
	int		status;		/* f77 return code */
	int		fl_libs;	/* add libs to command line */
	int		fl_show;	/* show what would be done */
	int		i;		/* favourite index */
	char		**av;		/* f77 arg vector */
	char		**sys;		/* system libraries arg vector */
	char		*phome;		/* installation dir */
	char		*tf77;		/* f77 tool */

	phome = getenv("LAMHOME");
	if (phome == 0) phome = getenv("TROLLIUSHOME");
	if (phome == 0) phome = DEFP;

	if ((strlen(phome) + 32) >= LAM_PATH_MAX) {
		fprintf(stderr, "hf77: LAM home pathname too long\n");
		exit(1);
	}
/*
 * Form include directory pathname.
 */
	strcpy(ph, "-I");
	strcat(ph, phome);
	strcat(ph, STRSDIR);
	strcat(ph, "h");
/*
 * Form library directory pathname.
 */
	strcpy(plib, FLIBPATH);
	strcat(plib, phome);
	strcat(plib, STRSDIR);
	strcat(plib, "lib");

	tf77 = getenv("LAMHF77");
	if (tf77 == 0) tf77 = getenv("TROLLIUSHF77");
	if (tf77 == 0) tf77 = DEFTF77;

	if ((av = sfh_argv_break(tf77, ' ')) == 0) {
		perror("hf77");
		exit(errno);
	}

	ac = sfh_argv_count(av);

	fl_show = 0;
	fl_libs = 1;

	for (i = 1; i < argc; ++i) {
		if (strcmp(argv[i], "-showme") == 0) {
			fl_show = 1;
		} else if (strcmp(argv[i], "-c") == 0) {
			fl_libs = 0;
		}
	}

	sfh_argv_add(&ac, &av, ph);
	if (fl_libs) {
		sfh_argv_add(&ac, &av, plib);
	}

	for (i = 1; i < argc; ++i) {
		if (strcmp(argv[i], "-showme")) {
			sfh_argv_add(&ac, &av, argv[i]);
		}
	}

	if (fl_libs) {
/*
 * Add system dependent link arguments.
 */
		if (strlen(HBIND) > 0) {
			if ((sys = sfh_argv_break(HBIND, ' ')) == 0) {
				perror("hcc");
				exit(errno);
			}

			for ( ; *sys; sys++) {
				sfh_argv_add(&ac, &av, *sys);
			}
		}
/*
 * Add standard LAM libraries.
 */
		sfh_argv_add(&ac, &av, "-ltstdio");
		sfh_argv_add(&ac, &av, "-ltrillium");
		sfh_argv_add(&ac, &av, "-largs");
		sfh_argv_add(&ac, &av, "-lt");
/*
 * Add system dependent libraries.
 */
		if (strlen(SYSLIBS) > 0) {
			if ((sys = sfh_argv_break(SYSLIBS, ' ')) == 0) {
				perror("hf77");
				exit(errno);
			}

			for ( ; *sys; sys++) {
				sfh_argv_add(&ac, &av, *sys);
			}
		}
	}

	if (fl_show) {
		for (i = 0; i < ac; i++) {
			printf("%s ", av[i]);
		}
		printf("\n");
	} else {
		status = cnfexec(av);

		if (status) {
			perror("hf77");
			exit(status);
		}
	}

	return(0);
}
